<?php
/**
 * This class contains static helper functions for console applications
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009, 2010 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ConsoleTools extends BaseFrameworkSystem {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Tries to resolve an IP address from given hostname. Currently only IPv
	 * addresses are resolved.
	 *
	 * @param	$hostname	Host name we shall resolve
	 * @return	$ip			IP address resolved from host name
	 * @todo	We should connect this to a caching class to cache DNS requests
	 */
	protected function resolveIpAddress ($hostname) {
		// Debug message
		$this->debugOutput(sprintf("[%s:] Our host name is: <span class=\"data\">%s</span>",
			$this->__toString(),
			$hostname
		));

		// Default is an invalid one
		$ip = '0.0.0.0';

		// Resolve it
		// @TODO Here should the cacher be implemented
		$ipResolved = gethostbyname($hostname);

		// Was it fine?
		if (($ipResolved !== false) && ($ipResolved != $hostname)) {
			// Okay, this works!
			$ip = $ipResolved;

			// Debug message
			$this->debugOutput(sprintf("[%s:] Resolved IP address is: <span class=\"data\">%s</span>\n",
				$this->__toString(),
				$ip
			));
		} else {
			// Problem while resolving IP address
			$this->debugOutput(sprintf("[%s:] Problem resolving IP address for host <span class=\"data\">%s</span>. Please check your /etc/hosts file.",
				$this->__toString(),
				$hostname
			));
		}

		// Return resolved IP
		return $ip;
	}

	/**
	 * Aquires the IP address of this host by reading the /etc/hostname file
	 * and solving it. It is now stored in configuration
	 *
	 * @return	void
	 */
	public static function acquireSelfIPAddress () {
		// Local IP by default
		$ip = '127.0.0.1';

		// Get a new instance
		$helper = new ConsoleTools();

		try {
			// Get a file pointer
			$io = FrameworkFileInputPointer::createFrameworkFileInputPointer('/etc/hostname');

			// Read the file
			$hostname = trim($io->readFromFile());

			// Close the file
			$io->closeFile();

			// Resolve the IP number
			$ip = $helper->resolveIpAddress($hostname);
		} catch (FileIoException $e) {
			// Fall-back to 'SESSION_SVR' which found on my Sun Station
			if (isset($_SERVER['SESSION_SVR'])) {
				// Resolve it
				$ip = $helper->resolveIpAddress($_SERVER['SESSION_SVR']);
			} else {
				// Could not find our hostname
				$helper->debugOutput(sprintf("[%s:] WARNING: Cannot resolve my own IP address.",
					$helper->__toString()
				));
			}
		} catch (FrameworkException $e) {
			// Output debug message
			$helper->debugOutput(sprintf("[%s:] Problem while resolving own IP address: [%s|%s]:%s",
				$helper->__toString(),
				$e->__toString(),
				$e->getHexCode(),
				$e->getMessage()
			));
		}

		// Set it in configuration
		FrameworkConfiguration::getInstance()->setServerAddress($ip);
	}

	/**
	 * Determines own remote IP address (e.g. can be used to probe if we are
	 * reachable from outside by determining external IP and then connect to it.
	 * This is accomblished by connecting to the IP of www.ship-simu.org which
	 * should default to 217.172.186.31. Of course, this method requires a valid
	 * and working Internet connection to work properly.
	 *
	 * This method is taken and lightly rewritten from a user comment on php.net:
	 * http://de.php.net/manual/en/function.socket-create.php#49368
	 *
	 * @param	$dest				IP destination we should connect to
	 * @param	$port				Port destination we should connect to
	 * @return	$externalAddress	The determined external IP address
	 * @throws	InvalidSocketException	If socket initialization wents wrong or if an errors occurs
	 */
	public function determineExternalIp ($dest = '217.172.186.31', $port = 80) {
		// First get a socket
		$socket = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);

		// Is it valid?
		if (!is_resource($socket)) {
			// Throw InvalidSocketException
			throw new InvalidSocketException (array($this, gettype($socket), 0, 'unknown'));
		} // END - if

		// Get socket error code for verification
		$socketError = socket_last_error($socket);

		// Check if there was an error else
		if ($socketError > 0) {
			// Then throw again
			throw new InvalidSocketException(array($this, gettype($socket), $socketError, socket_strerror($socketError)), BaseListener::EXCEPTION_INVALID_SOCKET);
		} // END - if

		// Connect to the destination
		socket_connect($socket, $dest, $port);

		// Get the socket address (our external IP) and port (ignored)
		socket_getsockname($socket, $externalAddress, $ourPort);

		// Close the socket
		socket_close($socket);

		// Return determined external IP
		return $externalAddress;
	}

	/**
	 * Analyzes the 'environment', mostly $_SERVER, for presence of elements
	 * which indicates clearly that e.g. this script has been executed from
	 * console or web.
	 *
	 * @return	$type	The analyzed type, can be 'http' or 'console'
	 */
	public static function analyzeEnvironmentForType () {
		// Default is the console
		$type = 'console';

		// Now, do we have a request method, or query string set?
		if ((isset($_SERVER['REQUEST_METHOD'])) || (isset($_SERVER['QUERY_STRING']))) {
			// Possibly HTTP request
			$type = 'http';
		} // END - if

		// Return it
		return $type;
	}

	/**
	 * Analyzes the 'environment', mostly $_SERVER, for presence of elements
	 * which indicates clearly that e.g. this script has been executed from
	 * console or web. This method should be used for class names, they
	 * currently are named differently. Here is a list to clarify this:
	 *
	 *   Request type | Class type
	 * -----------------------------
	 *      http      |    web
	 *     console    |  console
	 *
	 * @return	$type	The analyzed type, can be 'http' or 'console'
	 */
	public static function analyzeEnvironmentForClassType () {
		// Default is the console
		$type = 'console';

		// Now, do we have a request method, or query string set?
		if ((isset($_SERVER['REQUEST_METHOD'])) || (isset($_SERVER['QUERY_STRING']))) {
			// Possibly HTTP request
			$type = 'web';
		} // END - if

		// Return it
		return $type;
	}
}

// [EOF]
?>
